'''
\breif  This program is to find out the relationship between the frequency settings
    of Tx and Rx for SCuM.
    
\input  the freq_sweep result of tx and rx
    
\process
    tx: for result of each channel
            - select the data point from coarse 22-29
            - group the data point at each coarse value
            - calcuate the mean of frequency settings of each group
    rx: for result of each channel
            - select the data point from coarse 22-29
            - group the data point at each mid value
            - select two groups with largest data point size
            - calculate the mean of frequency settings of each group
            
\output
    1. 16x  2 frequency settings for each tx freq sweep result
    2. 16x  2 frequency settings for each rx freq sweep result
'''

import matplotlib 
import matplotlib.pyplot as plt
import numpy as np
import os
import json
import statistics
import operator

'''
output = {
    'tx': 
        {
            'channel_11':
                {
                    'coarse_1': [],
                    'coarse_2': [],
                    'coarse_1_center_setting': 0,
                    'coarse_2_center_setting': 0,
                }
            'channel_12':
                {
                    'coarse_1': [],
                    'coarse_2': [],
                    'coarse_1_center_setting': 0,
                    'coarse_2_center_setting': 0,
                }
            ...
        },
    'rx': 
        {
            'channel_11':
                {
                    'mid_1': [],
                    'mid_2': [],
                    'mid_1_center_setting': 0,
                    'mid_2_center_setting': 0,
                }
            'channel_12': [setting1, setting2],
            ...
        },
}
'''

output = {
    'tx': {},
    'rx': {}
}

coarse_range_s = 22
coarse_range_e = 29

def converter_config_linear_2_text(linear_configure):
    coarse = (linear_configure >> 10) & 0x1f
    mid = (linear_configure >> 5  )& 0x1f
    fine   = (linear_configure       )& 0x1f
    output = "{0}.{1}.{2}".format(coarse, mid, fine)
    return output

# input

with open('freq_sweep_tx_result.json') as jf:
    freq_sweep_tx_input = json.load(jf)
    for file_name, data in freq_sweep_tx_input.items():
        # skip result from rssi files (only channel 11-15 has rssi result)
            
        channel = file_name.split('.')[0].split('_')[-1]
        output['tx'][channel] = {}
        
        for config in data['linear_config']:
            coarse = config >> 10
            group = '{0}'.format(config>>10)
            if coarse >= coarse_range_s and coarse <= coarse_range_e:
                if group in output['tx'][channel].keys():
                    output['tx'][channel][group].append(config)
                else:
                    output['tx'][channel][group] = [config]
                    
    
with open('freq_sweep_rx_result.json') as jf:
    freq_sweep_rx_input = json.load(jf)
    for file_name, data in freq_sweep_rx_input.items():
        # skip result from rssi files (only channel 11-15 has rssi result)
        if 'rssi' in file_name:
            continue
        
        channel = file_name.split('.')[0].split('_')[-1]
        output['rx'][channel] = {}
        
        for config in data['linear_config']:
            coarse = config >> 10
            # group = '{0}.{1}'.format(config>>10, (config & 0b000001111100000)>>5)
            group = '{0}'.format(config>>10)
            if coarse >= coarse_range_s and coarse < coarse_range_e:
                if group in output['rx'][channel].keys():
                    output['rx'][channel][group].append(config)
                else:
                    output['rx'][channel][group] = [config]

# find the group which has lagest number of data points

result = {}

for type , data in output.items():
    print type
    for  channel, group_data in data.items():
        
        len_group = {}
        for group, freq in group_data.items():
            len_group[group] = len(freq)
        
        group_data['setting_1'] = 0
        group_data['setting_2'] = 0
        
        sorted_list = sorted(len_group.items(), key=operator.itemgetter(1), reverse=True)
        group_key_1 = sorted_list[0][0]
        group_data['setting_1'] = int(statistics.median(group_data[group_key_1]))
        
        if len(sorted_list)>1:
            group_key_2 = sorted_list[1][0]
            group_data['setting_2'] = int(statistics.median(group_data[group_key_2]))
        if not (channel in result):
            result[channel] = {}
        result[channel][type] = sorted([group_data['setting_1'], group_data['setting_2']])
    
# gather data in a new structure


sorted_result = {}
for channel, data in result.items():
    sorted_result[channel] = [
        data['tx'][0] - data['rx'][0], 
        data['tx'][0] - data['rx'][1], 
        data['tx'][1] - data['rx'][0], 
        data['tx'][1] - data['rx'][1]
    ]
    
for channel, value in sorted(sorted_result.items(), key=operator.itemgetter(0)):
    print channel, "{0:30} {1:20}".format(result[channel], value)
        
        
raw_input()